﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Data;

namespace SQLServerClient
{
    /// <summary>
    /// MainWindow.xaml 的交互逻辑
    /// </summary>
    public partial class MainWindow : Window
    {
        DBHelper db = new DBHelper(App.dbConnection.ToString());

        public MainWindow()
        {
            InitializeComponent();
        }

        ContextMenu menu_table = new ContextMenu();
        ContextMenu menu_view = new ContextMenu();
        ContextMenu menu_proc = new ContextMenu();
        ContextMenu menu_func = new ContextMenu();

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            this.Title += string.Format(" - 服务器 {0} 数据库 {1}"
                , App.dbConnection.Server
                , App.dbConnection.Database);

            InitTreeView();
            InitMenu();
        }
        #region 绑定数据库、操作菜单
        private void InitTreeView()
        { 
            List<TreeNode> nodes = new List<TreeNode>();

            nodes.Add(new TreeNode("数据表", 0, 0));
            nodes[0].ContextMenu = menu_table;
            FillChildNodes(nodes[0], TreeNodeType.Table);

            nodes.Add(new TreeNode("视图", 0, 0));
            nodes[1].ContextMenu = menu_view;
            FillChildNodes(nodes[1], TreeNodeType.View);

            nodes.Add(new TreeNode("存储过程", 0, 0));
            nodes[2].ContextMenu = menu_proc;
            FillChildNodes(nodes[2], TreeNodeType.Procedure);

            nodes.Add(new TreeNode("自定义函数", 0, 0));
            nodes[3].ContextMenu = menu_func;
            FillChildNodes(nodes[3], TreeNodeType.Function);

            nodes[0].IsExpanded = true;

            treeView1.ItemsSource = nodes;
        }
        private void InitMenu()
        {
            MenuItem item1 = new MenuItem();
            item1.Header = "查看表结构";
            item1.Click += new RoutedEventHandler(item1_Click);
            menu_table.Items.Add(item1);
            MenuItem item2 = new MenuItem();
            item2.Header = "生成Create脚本";
            item2.Click += new RoutedEventHandler(item2_Click);
            menu_table.Items.Add(item2);
            MenuItem item3 = new MenuItem();
            item3.Header = "生成Insert脚本";
            item3.Click += new RoutedEventHandler(item3_Click);
            menu_table.Items.Add(item3);
            
            MenuItem item4 = new MenuItem();
            item4.Header = "查看视图结构";
            item4.Click += new RoutedEventHandler(item4_Click);
            menu_view.Items.Add(item4);

            MenuItem item5 = new MenuItem();
            item5.Header = "查看脚本";
            item5.Click += new RoutedEventHandler(item5_Click);
            menu_proc.Items.Add(item5);

            MenuItem item6 = new MenuItem();
            item6.Header = "查看脚本";
            item6.Click += new RoutedEventHandler(item6_Click);
            menu_func.Items.Add(item6);
        }
        private void FillChildNodes(TreeNode node, TreeNodeType type)
        {
            string objType = "U";
            switch (type)
            {
                case TreeNodeType.Table:
                    objType = "U";
                    break;
                case TreeNodeType.View:
                    objType = "V";
                    break;
                case TreeNodeType.Procedure:
                    objType = "P";
                    break;
                case TreeNodeType.Function:
                    objType = "TF";
                    break;
            }
            string sql = string.Format(Sqls.get_objects, objType);
            DataTable dt = db.GetDataSet(sql).Tables[0];
            foreach (DataRow row in dt.Rows)
            {
                node.Items.Add(new TreeNode(row["name"].ToString(), type, 1));
            }
            textBlock1.Text += string.Format("{0}: {1} ", node.Header, dt.Rows.Count);
        }
        #endregion

        //执行SQL
        private void Button_Click(object sender, RoutedEventArgs e)
        {
            ExecuteSql();
        }
        private void ExecuteSql()
        {
            Print("正在执行。。。");
            string sql = textBox1.Text.Trim();
            if (sql.ToLower().StartsWith("select") || sql.ToLower().StartsWith("sp_help"))
            {
                DataSet ds = null;
                try
                {
                    ds = db.GetDataSet(sql);
                }
                catch (Exception ex)
                {
                    Print(ex.Message);
                    return;
                }
                BindResult(ds);
            }
            else
            {
                int i = 0;
                try
                {
                    i = db.Execute(sql);
                }
                catch (Exception ex)
                {
                    Print(ex.Message);
                    return;
                }
                Print("执行成功，影响行数：" + i);
            } 
        }
        #region 打印结果
        private void BindResult(DataSet ds)
        {
            dockPanel1.Children.Clear();
            TabControl tabCtl = new TabControl();
            foreach (DataTable dt in ds.Tables)
            {
                TabItem item = new TabItem();
                item.Header = dt.TableName;
                item.Content = BindGrid(dt);
                tabCtl.Items.Add(item);
            }
            dockPanel1.Children.Add(tabCtl);
        }
        private DataGrid BindGrid(DataTable dt)
        {
            DataGrid dg = new DataGrid();
            dg.IsReadOnly = true;
            dg.ItemsSource = dt.DefaultView;

            return dg;
        }
        private void Print(string msg)
        {
            dockPanel1.Children.Clear();
            TextBox txt = new TextBox();
            txt.Margin = new Thickness(0, 0, 0, 0);
            txt.IsReadOnly = true;
            txt.TextWrapping = TextWrapping.Wrap;
            txt.Text = msg;
            dockPanel1.Children.Add(txt);
        }
        #endregion

        //开启右键点击选中
        private void TreeViewItem_PreviewMouseRightButtonDown(object sender, MouseButtonEventArgs e)
        {
            //sender is TreeView
            TreeNode item = GetTemplatedAncestor<TreeNode>(e.OriginalSource as FrameworkElement);
            if (item != null) item.IsSelected = true;
        }

        private T GetTemplatedAncestor<T>(FrameworkElement element) where T : FrameworkElement
        {
            if (element is T)
            {
                return element as T;
            }

            FrameworkElement templatedParent = element.TemplatedParent as FrameworkElement;
            if (templatedParent != null)
            {
                return GetTemplatedAncestor<T>(templatedParent);
            }

            return null;
        }
        #region 右键菜单事件
        //查看表结构
        private void item1_Click(object sender, RoutedEventArgs e)
        {
            //var element = ContextMenuService.GetPlacementTarget(LogicalTreeHelper.GetParent(sender as MenuItem));
            //element is TreeView
            var node = GetSelectedNode(TreeNodeType.Table);
            if (node != null)
            {
                textBox1.Text = string.Format(Sqls.get_columns, node.Name);
                ExecuteSql();
            }
        }
        //生成create
        private void item2_Click(object sender, RoutedEventArgs e)
        {
            var node = GetSelectedNode(TreeNodeType.Table);
            if (node != null)
            {
                string tbName = node.Name;
                string sql = string.Format(Sqls.get_columns, tbName);
                DataTable dt = db.GetDataSet(sql).Tables[0];

                StringBuilder sb = new StringBuilder();
                sb.AppendFormat("CREATE TABLE [{0}](", tbName);
                //自动增长列
                sql = string.Format(Sqls.get_identity, tbName);
                DataTable dt2 = db.GetDataSet(sql).Tables[0];
                string identity = string.Empty;
                int seed = 1, increment = 1;
                object id_col = dt2.Rows[0]["Identity"];
                if (id_col != null && id_col.ToString() != "")
                {
                    identity = id_col.ToString();
                    seed = Convert.ToInt32(dt2.Rows[0]["Seed"]);
                    increment = Convert.ToInt32(dt2.Rows[0]["Increment"]);
                }
                //字段
                foreach (DataRow row in dt.Rows)
                {
                    sb.AppendFormat(" [{0}] {1}({2})", row["ColumnName"], row["Type"], row["length"]);
                    if (identity == row["ColumnName"].ToString())
                    {
                        sb.AppendFormat(" IDENTITY({0},{1})", seed, increment);
                    }
                    sb.AppendFormat(" {0} NULL,", (row["isnullable"].ToString() == "0" ? "NOT" : string.Empty));
                }
                sb.Remove(sb.Length - 1, 1);
                sb.Append(");\r\n");
                //约束
                sql = string.Format(Sqls.get_constraint, tbName);
                DataSet ds = db.GetDataSet(sql);
                if (ds.Tables.Count > 0)
                {
                    sb.AppendFormat("ALTER TABLE [{0}] WITH NOCHECK ADD", tbName);
                    foreach (DataRow row in ds.Tables[0].Rows)
                    {
                        sb.AppendFormat(" CONSTRAINT [{0}] ", row["constraint_name"]);
                        if (row["constraint_keys"].ToString().StartsWith("("))
                        {
                            sb.AppendFormat("DEFAULT {0} FOR [{1}],", row["constraint_keys"], row["constraint_type"].ToString().Split(' ')[3]);
                        }
                        else
                        {
                            sb.AppendFormat("{0} ([{1}]),", row["constraint_type"].ToString().Replace("(", "").Replace(")", ""), row["constraint_keys"]);
                        }
                    }
                    sb.Remove(sb.Length - 1, 1);
                    sb.Append(";\r\n");
                }
                //索引
                sql = string.Format(Sqls.get_index, tbName);
                ds = db.GetDataSet(sql);
                if (ds.Tables.Count > 0)
                {
                    foreach (DataRow row in ds.Tables[0].Rows)
                    {
                        sb.AppendFormat("CREATE INDEX [{0}] ON [{1}] ({2});\r\n", row["index_name"], tbName, row["index_keys"]);
                    }
                }
                textBox1.Text = sb.ToString();

                dockPanel1.Children.Clear();
            }
        }
        //生成insert
        private void item3_Click(object sender, RoutedEventArgs e)
        {
            var node = GetSelectedNode(TreeNodeType.Table);
            if (node != null)
            {
                string sql = string.Format(Sqls.get_insert, node.Name);
                DataSet ds = db.GetDataSet(sql);
                StringBuilder sb = new StringBuilder();
                foreach (DataTable dt in ds.Tables)
                {
                    foreach (DataRow row in dt.Rows)
                    {
                        for (int i = 0; i < dt.Columns.Count; i++)
                        {
                            sb.Append(row[i].ToString());
                        }
                        sb.Append(";\r\n");
                    }
                }
                textBox1.Text = sb.ToString();
                dockPanel1.Children.Clear();
            }
        }
        //查看视图结构
        private void item4_Click(object sender, RoutedEventArgs e)
        {
            var node = GetSelectedNode(TreeNodeType.View);
            if (node != null)
            {
                textBox1.Text = string.Format(Sqls.get_columns, node.Name);
                ExecuteSql();
            }
        }
        //存储过程脚本
        private void item5_Click(object sender, RoutedEventArgs e)
        {
            var node = GetSelectedNode(TreeNodeType.Procedure);
            if (node != null)
            {
                string sql = string.Format(Sqls.get_text, node.Name);
                DataTable dt = db.GetDataSet(sql).Tables[0];
                StringBuilder sb = new StringBuilder();
                foreach (DataRow row in dt.Rows)
                {
                    sb.AppendFormat("{0}", row[0]);
                }
                textBox1.Text = sb.ToString();
                dockPanel1.Children.Clear();
            }
        }
        //自定义函数脚本
        private void item6_Click(object sender, RoutedEventArgs e)
        {
            var node = GetSelectedNode(TreeNodeType.Function);
            if (node != null)
            {
                string sql = string.Format(Sqls.get_text, node.Name);
                DataTable dt = db.GetDataSet(sql).Tables[0];
                StringBuilder sb = new StringBuilder();
                foreach (DataRow row in dt.Rows)
                {
                    sb.AppendFormat("{0}", row[0]);
                }
                textBox1.Text = sb.ToString();
                dockPanel1.Children.Clear();
            }
        }
        private TreeNode GetSelectedNode(TreeNodeType type)
        {
            var node = treeView1.SelectedItem as TreeNode;
            if (node == null || node.NodeType != type)
            {
                MessageBox.Show("没有选择正确的对象。");
                return null;
            }
            return node;
        }
        #endregion
    }
}
